/* dumb-init.c
 * $Id: dumb-init.c,v 1.4 1998/07/08 03:45:40 al Exp $
 *
 * Copyright 1997,1998 Alva Petrofsky <alva@petrofsky.berkeley.ca.us>.
 * Any use permitted provided this notice stays intact.
 */

#include "dumb-frotz.h"
#include "gp32.h"
#include "fileio.h"
#include "sbox.h"

static char usage[] = "\
\n\
FROTZ V2.32 - interpreter for all Infocom games. Complies with standard\n\
1.0 of Graham Nelson's specification. Written by Stefan Jokisch in 1995-7.\n\
\n\
DUMB-FROTZ V2.32R1 - port for all platforms.  Somewhat complies with standard\n\
9899 of ISO's specification.  Written by Alembic Petrofsky in 1997-8.\n\
\n\
Syntax: frotz [options] story-file [graphics-file]\n\
\n\
  -a      watch attribute setting\n\
  -A      watch attribute testing\n\
  -h #    screen height\n\
  -i      ignore runtime errors\n\
  -I #    interpreter number to report to game\n\
  -o      watch object movement\n\
  -O      watch object locating\n\
  -p      alter piracy opcode\n\
  -P      transliterate latin1 to plain ASCII\n\
  -R xxx  do runtime setting \\xxx before starting\n\
            (this option can be used multiple times)\n\
  -s #    random number seed value\n\
  -S #    transscript width\n\
  -t      set Tandy bit\n\
  -u #    slots for multiple undo\n\
  -w #    screen width\n\
  -x      expand abbreviations g/x/z\n\
\n\
While running, enter \"\\help\" to list the runtime escape sequences.\n\
";

static int user_screen_width = 75;
static int user_screen_height = 24;
static int user_interpreter_number = -1;
static int user_random_seed = -1;
static int user_tandy_bit = 0;
static char *graphics_filename = NULL;
static bool plain_ascii = FALSE;

extern unsigned short *framebuffer;
extern void print_buffer();
extern DIR dir_struct;
extern int dir_entrys;

void os_process_arguments(int argc, char *argv[]) 
{
  
/*
	  case 'a': option_attribute_assignment = 1; break;
	  case 'A': option_attribute_testing = 1; break;
      case 'h': user_screen_height = atoi(zoptarg); break;
	  case 'i': option_ignore_errors = 1; break;
	  case 'I': user_interpreter_number = atoi(zoptarg); break;
	  case 'o': option_object_movement = 1; break;
	  case 'O': option_object_locating = 1; break;
	  case 'p': option_piracy = 1; break;
	  case 'P': plain_ascii = 1; break;
	  case 'R': dumb_handle_setting(zoptarg, FALSE, TRUE); break;
	  case 's': user_random_seed = atoi(zoptarg); break;
	  case 'S': option_script_cols = atoi(zoptarg); break;
	  case 't': user_tandy_bit = 1; break;
	  case 'u': option_undo_slots = atoi(zoptarg); break;
	  case 'w': user_screen_width = atoi(zoptarg); break;
	  case 'x': option_expand_abbreviations = 1; break;
	*/

	int button;
	
	printstring("Reading gpmm\\frotz directory");

      dir_entrys=smc_dir("dev0:\\gpmm\\FROTZ",&dir_struct);

//    sbox.textcolor = 0xF800>>5;   // Green
//    sbox.boxcolor  = 0xF800>>10;  // Blue

	char buff[32];

    int ds,dx,dy,i,j,k,sx,ex;
	int select,scroll,entry;

	unsigned short fg;
	unsigned short bg;

	select=0;
	scroll=0;
	
	/* A och y har bytt plats!*/
	sx=240*47;
	for(i=6;i<233;i++){
	     framebuffer[sx+i]=0;
	}
	sx=240*152;
	for(i=6;i<233;i++){
	     framebuffer[sx+i]=0;
	}
	sx=240*153;
	for(i=6;i<233;i++){
	     framebuffer[sx+i]=0;
	}

	for(i=6;i<8;i++){
		for(j=47;j<153;j++){
	     framebuffer[(j*240)+i]=0;		    
		}
	}
	i=232;
	for(j=47;j<153;j++){
	     framebuffer[(j*240)+i]=0;		    
	}


	do{     

      entry=scroll;

	  dx=6;
	  dy=1;

	  for(i=1;i<29;i++){
        
		if(entry<(dir_entrys-2)){
			sprintf(buff,"%s             ",dir_struct.name[entry+2]);
		}else{
			sprintf(buff,"                  ");
		}

		if (entry==select){
			bg=0xF800>>10;
			fg=0xFFFF;
		}else{
			bg=(0xEC00)|(0xEC00>>5)|(0xF800>>10);		
			fg=0xF800>>10;
		}
        
		sx=((29-dy)*8);
	    ex=((29-dy)*8)+8;
	    for(k=(dx*8);k<((dx*8)+104);k++){
     	    for(j=sx;j<ex;j++){
			    framebuffer[(k*240)+j]=bg;
		    }
	    }

	    gp_drawString(dx*8,dy*8,13,buff,fg,framebuffer);

		dy++;
		entry++;

	  }

	  do{
	       button=gp_getButton();
	  }while(button==0);

	  if (button&BUTTON_DOWN){
		  if (select<(dir_entrys-3)) select++;
	  }
	  if (button&BUTTON_UP){
		  if (select>0) select--;
	  }

	  if(select<scroll) scroll=select;
	  if(select-27>scroll) scroll=select-27;

	}while(!(button&BUTTON_A));

	static char storyname[40];
	
	strcpy(storyname,"dev0:\\GPMM\\FROTZ\\");
	if((select+2)<dir_entrys){
		story_name=strcat(storyname,dir_struct.name[select+2]);
	}else{
		strcat(storyname,"<no file>");
	}
    print_buffer();

	user_screen_width=40;
	user_screen_height=29;
	option_piracy = 1;

}

void os_init_screen(void)
{
  if (h_version == V3 && user_tandy_bit)
      h_config |= CONFIG_TANDY;

  if (h_version >= V5 && option_undo_slots == 0)
      h_flags &= ~UNDO_FLAG;

  h_screen_rows = user_screen_height;
  h_screen_cols = user_screen_width;

  if (user_interpreter_number > 0)
    h_interpreter_number = user_interpreter_number;
  else {
    /* Use ms-dos for v6 (because that's what most people have the
     * graphics files for), but don't use it for v5 (or Beyond Zork
     * will try to use funky characters).  */
    h_interpreter_number = h_version == 6 ? INTERP_MSDOS : INTERP_DEC_20;
  }
  h_interpreter_version = 'F';

  dumb_init_input();
  dumb_init_output();
  dumb_init_pictures(graphics_filename);
}

int os_random_seed (void)
{
  if (user_random_seed == -1)
    /* Use the epoch as seed value */
    return (time(0) & 0x7fff);
  else return user_random_seed;
}

void os_restart_game (int stage) {}

void os_fatal (const char *s)
{
  char buffer[1024];

  sprintf(buffer,"Fatal error: %s\n",s);
  printstring(buffer);
  
  while(1);
}
