/*
 * main.c
 *
 * Frotz V2.32 main function
 *
 * This is an interpreter for Infocom V1 to V6 games. It also supports
 * the recently defined V7 and V8 games. Please report bugs to
 *
 *    s.jokisch@avu.de
 *
 * Frotz is freeware. It may be used and distributed freely provided
 * no commercial profit is involved. (c) 1995-1997 Stefan Jokisch
 *
 */

#include "frotz.h"
#include "gp32.h"
#include "fileio.h"

#ifndef __MSDOS__
#define cdecl
#endif

#include "gp32.h"

extern void interpret (void);
extern void init_memory (void);
extern void init_undo (void);
extern void reset_memory (void);

void printstring(char *instring);

unsigned short *framebuffer;
char printbuffer[29][40];      // Screen buffer
char charbuffer[2];
int cx,cy,sx;                     // Cursor X,Y, Status Cursor X 
char wordbuffer[40];
int wordpos;
int linecount=0;
/* Story file name, id number and size */

const char *story_name = 0;

enum story story_id = UNKNOWN;
long story_size = 0;

/* Story file header data */

zbyte h_version = 0;
zbyte h_config = 0;
zword h_release = 0;
zword h_resident_size = 0;
zword h_start_pc = 0;
zword h_dictionary = 0;
zword h_objects = 0;
zword h_globals = 0;
zword h_dynamic_size = 0;
zword h_flags = 0;
zbyte h_serial[6] = { 0, 0, 0, 0, 0, 0 };
zword h_abbreviations = 0;
zword h_file_size = 0;
zword h_checksum = 0;
zbyte h_interpreter_number = 0;
zbyte h_interpreter_version = 0;
zbyte h_screen_rows = 0;
zbyte h_screen_cols = 0;
zword h_screen_width = 0;
zword h_screen_height = 0;
zbyte h_font_height = 1;
zbyte h_font_width = 1;
zword h_functions_offset = 0;
zword h_strings_offset = 0;
zbyte h_default_background = 0;
zbyte h_default_foreground = 0;
zword h_terminating_keys = 0;
zword h_line_width = 0;
zbyte h_standard_high = 1;
zbyte h_standard_low = 0;
zword h_alphabet = 0;
zword h_extension_table = 0;
zbyte h_user_name[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };

zword hx_table_size = 0;
zword hx_mouse_x = 0;
zword hx_mouse_y = 0;
zword hx_unicode_table = 0;

/* Stack data */

zword stack[STACK_SIZE];
zword *sp = 0;
zword *fp = 0;

/* IO streams */

bool ostream_screen = TRUE;
bool ostream_script = FALSE;
bool ostream_memory = FALSE;
bool ostream_record = FALSE;
bool istream_replay = FALSE;
bool message = FALSE;

/* Current window and mouse data */

int cwin = 0;
int mwin = 0;

int mouse_y = 0;
int mouse_x = 0;

/* Window attributes */

bool enable_wrapping = FALSE;
bool enable_scripting = FALSE;
bool enable_scrolling = FALSE;
bool enable_buffering = FALSE;

/* User options */

int option_attribute_assignment = 0;
int option_attribute_testing = 0;
int option_context_lines = 0;
int option_object_locating = 0;
int option_object_movement = 0;
int option_left_margin = 0;
int option_right_margin = 0;
int option_ignore_errors = 0;
int option_piracy = 0;
int option_undo_slots = MAX_UNDO_SLOTS;
int option_expand_abbreviations = 0;
int option_script_cols = 80;

/* Size of memory to reserve (in bytes) */

long reserve_mem = 0;

/*
 * runtime_error
 *
 * An error has occured. Ignore it or pass it to os_fatal.
 *
 */

void runtime_error (const char *s)
{

    if (!option_ignore_errors)
	{ flush_buffer (); os_fatal (s); }

}/* runtime_error */

/*
 * z_piracy, branch if the story file is a legal copy.
 *
 *	no zargs used
 *
 */

void z_piracy (void)
{

    branch (!option_piracy);

}/* z_piracy */

/*
 * main
 *
 * Prepare and run the game.
 *
 */

/*
  
I dumped GP32 specific stuff here in order to keep it all in one place!

*/

void print_buffer()
{
	int i,j,k;
	int sx,ex,fk;
	for(j=1;j<30;j++){

		for(k=0;k<320;k++){
            sx=((29-j)*8);
			ex=((29-j)*8)+8;
			fk=k*240;
			for(i=sx;i<ex;i++){
			     framebuffer[fk+i]=0xffff;
			}
		}

		if (j<29) gp_drawString(0,j*8,40,printbuffer[j],0x0000,framebuffer);
	}
}

void 	flush_buffer()
{
       /* Does nothing, printing is done on the fly in v1.0 */
}

void init_buffer()
{
	     int i,j;
	     cx=0;
		 cy=1;
	     for(i=0;i<40;i++){
			 for(j=0;j<29;j++){
			      printbuffer[j][i]=' ';
			 }
		 }
		 print_buffer();

		 wordpos=0;
}

void showcursor(int status)
{
    int i,j,sx,ex;

    sx=((29-cy)*8);
	ex=((29-cy)*8)+8;
	for(i=(cx*8);i<((cx*8)+8);i++){
     	for(j=sx;j<ex;j++){
			if(status==1) framebuffer[(i*240)+j]=0xf800;
			if(status==0) framebuffer[(i*240)+j]=0xffff;
		}
	}

}

void	new_line()
{
	 // If new line before end of line, pad with spaces
	 int i,j; 
	 if (cx<=39){
		 for(i=cx;i<40;i++){
			 printbuffer[cy][i]=' ';
		 }
	 }
	 
	 cx=0;
	 cy=cy+1;
	 if (cy==29){
		 for(i=0;i<40;i++){
			 for(j=1;j<28;j++){
			      printbuffer[j][i]=printbuffer[j+1][i];
			 }
		 }

		 for(i=0;i<40;i++){
			 printbuffer[28][i]=' ';
		 }

		 cy=28;

		 print_buffer();
	 }
     linecount++;
     
	 if(linecount==26){
	     new_line();  

		 printstring("<<PRESS ANY KEY>>");
		 linecount=0;

		 int curson=0;
		 char get=0;
 
         while(get==0){

		    int x;
		    for (x=0;x<1000000;x++) x=x;   // Some waiting
        
			get=gp_getChatboard();
       
			if(curson){
			     curson=0;
				 showcursor(1);
			}else{
			     curson=1;
				 showcursor(0);
			}
		}
    	
		 showcursor(0);
	 }
}

void	print_char (zchar inchar)
{
    int i,j,sx,ex;

	if(inchar==' '&&cx==0) return;

	if(cx==40){
	   new_line(); 
	}
	
	printbuffer[cy][cx]=inchar;
	charbuffer[0]=inchar;
	charbuffer[1]=0;
	
    sx=((29-cy)*8);
	ex=((29-cy)*8)+8;
	for(i=(cx*8);i<((cx*8)+8);i++){
     	for(j=sx;j<ex;j++){
			framebuffer[(i*240)+j]=0xffff;
		}
	}

	gp_drawString(cx*8,cy*8,1,charbuffer,0x0000,framebuffer);		
	cx=cx+1;

//	print_buffer();
}

void clear_status()
{
	sx=0;
}


void print_statuschar(zchar inchar)
{
    int i,j,ex;

	if(sx<=39){ 
		printbuffer[0][sx]=inchar;
		charbuffer[0]=inchar;
		charbuffer[1]=0;
	
		for(i=(sx*8);i<((sx*8)+8);i++){
     		for(j=232;j<240;j++){
				framebuffer[(i*240)+j]=0xf800;
			}
		}

		gp_drawString(sx*8,0,1,charbuffer,0xffff,framebuffer);		
		sx=sx+1;
	}

//	print_buffer();
}

void printstatusstring(char *instring)
{
	int i;
    for(i=0;i<strlen(instring);i++){
        print_statuschar(instring[i]);
	}
}

void printstring(char *instring)
{
	int i;
    for(i=0;i<strlen(instring);i++){
        print_char(instring[i]);
	}
	new_line();
}

void printstringo(char *instring)
{
	int i;
    for(i=0;i<strlen(instring);i++){
        print_char(instring[i]);
	}
}

void flushword()
{
    wordbuffer[wordpos]=0;
	if((cx+strlen(wordbuffer))>40){
	       new_line();
		   printstringo(wordbuffer);    
	}else{
		   printstringo(wordbuffer);
	}
	wordpos=0;
}

void addword(char c)
{
     wordbuffer[wordpos]=c;
	 wordpos++;
}

char *filterstring(int no, char *s){
     static char t[128];

     int i,j;

	 j=0;

	 for(i=0;i<no;i++){
		 if(s[i]==10){
		 }else if(s[i]==9){
		      t[j]=' ';
			  j++;		 		      
		 }else if(s[i]==13){
		      t[j]=' ';
			  j++;		 		      
		 }else{
		      t[j]=s[i];
			  j++;		 
		 }
	 }

	 t[j]=0;
	 
	 return t;
}


DIR dir_struct;
int dir_entrys;

void showtextfile()
{
	printstring("Reading gpmm\\docs directory");

    dir_entrys=smc_dir("dev0:\\gpmm\\DOCS",&dir_struct);

//    sbox.textcolor = 0xF800>>5;   // Green
//    sbox.boxcolor  = 0xF800>>10;  // Blue

	char buff[32];

    int ds,dx,dy,i,j,k,sx,ex;
	int select,scroll,entry;

	unsigned short fg;
	unsigned short bg;

	select=0;
	scroll=0;
	
	volatile int button;

	/* A och y har bytt plats!*/
	sx=240*47;
	for(i=6;i<233;i++){
	     framebuffer[sx+i]=0;
	}
	sx=240*152;
	for(i=6;i<233;i++){
	     framebuffer[sx+i]=0;
	}
	sx=240*153;
	for(i=6;i<233;i++){
	     framebuffer[sx+i]=0;
	}

	for(i=6;i<8;i++){
		for(j=47;j<153;j++){
	     framebuffer[(j*240)+i]=0;		    
		}
	}
	i=232;
	for(j=47;j<153;j++){
	     framebuffer[(j*240)+i]=0;		    
	}


	do{     

      entry=scroll;

	  dx=6;
	  dy=1;

	  for(i=1;i<29;i++){
        
		if(entry<(dir_entrys-2)){
			sprintf(buff,"%s             ",dir_struct.name[entry+2]);
		}else{
			sprintf(buff,"                  ");
		}

		if (entry==select){
			bg=0xF800>>10;
			fg=0xFFFF;
		}else{
			bg=(0xEC00)|(0xEC00>>5)|(0xF800>>10);		
			fg=0xF800>>10;
		}
        
		sx=((29-dy)*8);
	    ex=((29-dy)*8)+8;
	    for(k=(dx*8);k<((dx*8)+104);k++){
     	    for(j=sx;j<ex;j++){
			    framebuffer[(k*240)+j]=bg;
		    }
	    }

	    gp_drawString(dx*8,dy*8,13,buff,fg,framebuffer);

		dy++;
		entry++;

	  }

	do{
	       button=gp_getButton();
	  }while(button==0);

	  if (button&BUTTON_DOWN){
		  if (select<(dir_entrys-3)) select++;
	  }
	  if (button&BUTTON_UP){
		  if (select>0) select--;
	  }

	  if(select<scroll) scroll=select;
	  if(select-27>scroll) scroll=select-27;

	}while(!(button&BUTTON_A));

	static char storyname[40];
	
	strcpy(storyname,"dev0:\\GPMM\\DOCS\\");
	if((select+2)<dir_entrys){
		story_name=strcat(storyname,dir_struct.name[select+2]);
	}else{
		strcat(storyname,"<no file>");
	}

	GPFILE *file;
	int offs;
	char buf[80];
	char *space;
	int wordpos,wordstart;
	int filepos;
	int ccx;
	int *filindex;
    int lc;

	file=smc_fopen(storyname,"rb");
	smc_fseek(file,0,SEEK_END);
	offs=smc_ftell(file);

	space=malloc(offs+16);
    
	if(space==0){
		printstring("Fatal: Not enough memory!");
    	while(1){}
	}

	smc_fseek(file,0,SEEK_SET);
	smc_fread(space,1,offs,file);
	smc_fclose(file);

	lc=0;
    wordpos=0;
	filepos=0;
	ccx=0;
	wordstart=0;

	// Wordpos is the position within the word

	for(i=0;i<offs;i++){

		if (space[i]==' '||space[i]==9) {

            // Here we know that wordpos is lower than 40 so at most one line!
			if((wordpos>0)&&(space[i]==' '||space[i]==9)){
			   if(ccx+wordpos>37){
			      lc++;
				  ccx=wordpos;
//			      print_char('S');
			   }else{
			      ccx+=wordpos;
//			      print_char('O');
			   }
			}else{
//			      print_char('N');			
			}

			ccx++;
			if(ccx==38){
			    lc++;
				ccx=0;

//   			    print_char('R');
			}
			
			wordpos=0;
		}else if(space[i]==10){
			if(ccx+wordpos>37){
				lc++;
				lc++;
				wordstart=i+1;
//			    print_char('D');				  
			}else{
			    lc++;
				wordstart=i+1;
//			    print_char('E');				  
			}
			
			wordpos=0;
			ccx=0;
		}else{
		       wordpos++;
//   			   print_char('V');
		}

		if (wordpos==38){
			lc++;
		    lc++;

			wordstart=i+1;
					
			ccx=0;				         
			wordpos=0;

//   			print_char('X');
		}

//    	sprintf(buf,"%c %i %i %i %i",space[i],wordpos,ccx,wordstart,lc,i);
//		printstring(buf);

	}

	filindex=malloc(sizeof(int)*(lc+3));
    filindex[0]=0;

	lc=0;
    wordpos=0;
	filepos=0;
	ccx=0;
	wordstart=0;

	for(i=0;i<offs;i++){

		if (space[i]==' '||space[i]==9) {

            // Here we know that wordpos is lower than 40 so at most one line!
			if((wordpos>0)&&(space[i]==' '||space[i]==9)){
			   if(ccx+wordpos>37){
			      lc++;
				  filindex[lc]=wordstart;
				  ccx=wordpos;
//			      print_char('S');
  		          wordstart=i-wordpos;
			   }else{
			      ccx+=wordpos;
//			      print_char('O');
			   }
			}else{
//			      print_char('N');			
			}

			ccx++;
			if(ccx==38){
			    lc++;
				filindex[lc]=wordstart;
				ccx=0;
// 			    print_char('R');
    		    wordstart=i+1;

			}
			
			wordpos=0;

		}else if(space[i]==10){
			if(ccx+wordpos>37){
				lc++;
				filindex[lc]=wordstart;

				lc++;
				filindex[lc]=i-wordpos;
				wordstart=i+1;				

//			    print_char('D');				  
			}else{
			    lc++;
				filindex[lc]=wordstart;
				wordstart=i+1;
//			    print_char('E');				  
			}
			
			wordpos=0;
			ccx=0;
		}else{
		       wordpos++;
//   			   print_char('V');
		}

//    	sprintf(buf,"D%c%c%c%c%c%c",space[wordstart],space[wordstart+1],space[wordstart+2],space[wordstart+3],space[wordstart+4],space[wordstart+5]);
//		printstring(buf);



		if (wordpos==38){
			lc++;
    		filindex[lc]=wordstart;

		    lc++;
    		filindex[lc]=i-wordpos;

			wordstart=i+1;
					
			ccx=0;				         
			wordpos=0;

//   			print_char('X');
		}

//    	sprintf(buf,"%c %i %i %i %i",space[i],wordpos,ccx,wordstart,lc,i);
//		printstring(buf);

	}

	lc++;
	filindex[lc]=offs;
	lc--;

	scroll=0;

	do{

		int k,sx,ex,fk;
		char *filtered;
		char *nothing="                                        ";
		for(j=0;j<29;j++){
			
			for(k=0;k<320;k++){
				sx=((29-j)*8);
				ex=((29-j)*8)+8;
				fk=k*240;
				for(i=sx;i<ex;i++){
					framebuffer[fk+i]=0xffff;
				}
			}
			
			if(j+scroll<lc+1){
				wordstart=filindex[j+scroll];
				wordpos=filindex[j+scroll+1]-filindex[j+scroll];

				for(k=0;k<320;k++){
					sx=((29-j)*8);
					ex=((29-j)*8)+8;
					fk=k*240;
					for(i=sx;i<ex;i++){
						framebuffer[fk+i]=0xffff;
					}
				}

				filtered=filterstring(wordpos,space+wordstart);
				gp_drawString(0,j*8,strlen(filtered),filtered,0x0000,framebuffer);
			}
		}

		do{
	       button=gp_getButton();
		}while(button==0);

		if (button&BUTTON_DOWN){
			if (scroll<lc) scroll++;
		}
		if (button&BUTTON_UP){
			if (scroll>0) scroll--;
		}

		if (button&BUTTON_R){
			scroll+=20;
			if (scroll>lc) scroll=lc;
		}
		if (button&BUTTON_L){
			scroll-=20;
			if(scroll<0) scroll=0;
		}

	}while(!(button&BUTTON_START));

    free(filindex);
	free(space);

	print_buffer();
}

int cdecl main (int argc, char *argv[])
{ 
    framebuffer = (u16*) FRAMEBUFFER;

    gp_setCpuspeed(33);  
    gp_initFramebuffer(framebuffer,16,85);
    gp_clearFramebuffer16(framebuffer,0xFFFF); // Clear Screen

	// Text viewer test!
	
	init_buffer();
    print_buffer();

	printstring("GP32 Initiation Complete");
	printstring("Initiating Chatboard");

    int init = gp_initChatboard();


//    DIR dir_struct;
//    int dir_entrys;
	
	print_buffer();

    if (init) {
	    printstring("Chatboard initiated!");	
	}else{
	    printstring("Chatboard not found!");
	}

    os_process_arguments (argc, argv);

	init_memory();
    os_init_screen ();
	init_undo ();
    z_restart ();

	printstring("Z Restarted");

    interpret ();

	printstring("Z Interpretation Finished");

    reset_memory ();

    os_reset_screen ();

	printstring("Thats all folks!");

}/* main */
